from typing import Any, Optional

from botocore import utils as utils

BOTOCORE_DEFAUT_SESSION_VARIABLES: Any
DEFAULT_S3_CONFIG_VARS: Any
DEFAULT_PROXIES_CONFIG_VARS: Any

def create_botocore_default_config_mapping(session: Any) -> Any: ...

class ConfigChainFactory:
    def __init__(self, session: Any, environ: Optional[Any] = ...) -> None: ...
    def create_config_chain(
        self,
        instance_name: Optional[Any] = ...,
        env_var_names: Optional[Any] = ...,
        config_property_names: Optional[Any] = ...,
        default: Optional[Any] = ...,
        conversion_func: Optional[Any] = ...,
    ) -> Any: ...

class ConfigValueStore:
    def __init__(self, mapping: Optional[Any] = ...) -> None: ...
    def get_config_variable(self, logical_name: Any) -> Any: ...
    def set_config_variable(self, logical_name: Any, value: Any) -> None: ...
    def clear_config_variable(self, logical_name: Any) -> None: ...
    def set_config_provider(self, logical_name: Any, provider: Any) -> None: ...

class BaseProvider:
    def provide(self) -> None: ...

class ChainProvider(BaseProvider):
    def __init__(
        self, providers: Optional[Any] = ..., conversion_func: Optional[Any] = ...
    ) -> None: ...
    def provide(self) -> Any: ...

class InstanceVarProvider(BaseProvider):
    def __init__(self, instance_var: Any, session: Any) -> None: ...
    def provide(self) -> Any: ...

class ScopedConfigProvider(BaseProvider):
    def __init__(self, config_var_name: Any, session: Any) -> None: ...
    def provide(self) -> Any: ...

class EnvironmentProvider(BaseProvider):
    def __init__(self, name: Any, env: Any) -> None: ...
    def provide(self) -> Any: ...

class SectionConfigProvider(BaseProvider):
    def __init__(
        self, section_name: Any, session: Any, override_providers: Optional[Any] = ...
    ) -> None: ...
    def provide(self) -> Any: ...

class ConstantProvider(BaseProvider):
    def __init__(self, value: Any) -> None: ...
    def provide(self) -> Any: ...



from typing import Any, Optional

from botocore.compat import ensure_bytes as ensure_bytes
from botocore.compat import ensure_unicode as ensure_unicode
from botocore.compat import urlparse as urlparse

class Monitor:
    def __init__(self, adapter: Any, publisher: Any) -> None: ...
    def register(self, event_emitter: Any) -> None: ...
    def capture(self, event_name: Any, **payload: Any) -> None: ...

class MonitorEventAdapter:
    def __init__(self, time: Any = ...) -> None: ...
    def feed(self, emitter_event_name: Any, emitter_payload: Any) -> Any: ...

class BaseMonitorEvent:
    service: Any = ...
    operation: Any = ...
    timestamp: Any = ...
    def __init__(self, service: Any, operation: Any, timestamp: Any) -> None: ...
    def __eq__(self, other: Any) -> Any: ...

class APICallEvent(BaseMonitorEvent):
    latency: Any = ...
    attempts: Any = ...
    retries_exceeded: Any = ...
    def __init__(
        self,
        service: Any,
        operation: Any,
        timestamp: Any,
        latency: Optional[Any] = ...,
        attempts: Optional[Any] = ...,
        retries_exceeded: bool = ...,
    ) -> None: ...
    def new_api_call_attempt(self, timestamp: Any) -> Any: ...

class APICallAttemptEvent(BaseMonitorEvent):
    latency: Any = ...
    url: Any = ...
    http_status_code: Any = ...
    request_headers: Any = ...
    response_headers: Any = ...
    parsed_error: Any = ...
    wire_exception: Any = ...
    def __init__(
        self,
        service: Any,
        operation: Any,
        timestamp: Any,
        latency: Optional[Any] = ...,
        url: Optional[Any] = ...,
        http_status_code: Optional[Any] = ...,
        request_headers: Optional[Any] = ...,
        response_headers: Optional[Any] = ...,
        parsed_error: Optional[Any] = ...,
        wire_exception: Optional[Any] = ...,
    ) -> None: ...

class CSMSerializer:
    csm_client_id: Any = ...
    def __init__(self, csm_client_id: Any) -> None: ...
    def serialize(self, event: Any) -> Any: ...

class SocketPublisher:
    def __init__(self, socket: Any, host: Any, port: Any, serializer: Any) -> None: ...
    def publish(self, event: Any) -> None: ...



from typing import Any, Optional

from botocore.compat import HTTPHeaders as HTTPHeaders
from botocore.compat import HTTPResponse as HTTPResponse
from botocore.compat import MutableMapping as MutableMapping
from botocore.compat import urlencode as urlencode
from botocore.compat import urlsplit as urlsplit
from botocore.compat import urlunsplit as urlunsplit
from botocore.exceptions import UnseekableStreamError as UnseekableStreamError
from requests.packages.urllib3.connection import HTTPConnection, VerifiedHTTPSConnection
from requests.packages.urllib3.connectionpool import HTTPConnectionPool, HTTPSConnectionPool

class AWSHTTPResponse(HTTPResponse):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class AWSConnection:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    response_class: Any = ...
    def close(self) -> None: ...
    def send(self, str: Any) -> Any: ...

class AWSHTTPConnection(AWSConnection, HTTPConnection): ...
class AWSHTTPSConnection(AWSConnection, VerifiedHTTPSConnection): ...

class AWSHTTPConnectionPool(HTTPConnectionPool):
    ConnectionCls: Any = ...

class AWSHTTPSConnectionPool(HTTPSConnectionPool):
    ConnectionCls: Any = ...

def prepare_request_dict(
    request_dict: Any,
    endpoint_url: Any,
    context: Optional[Any] = ...,
    user_agent: Optional[Any] = ...,
) -> None: ...
def create_request_object(request_dict: Any) -> Any: ...

class AWSRequestPreparer:
    def prepare(self, original: Any) -> Any: ...

class AWSRequest:
    method: Any = ...
    url: Any = ...
    headers: Any = ...
    data: Any = ...
    params: Any = ...
    auth_path: Any = ...
    stream_output: Any = ...
    context: Any = ...
    def __init__(
        self,
        method: Optional[Any] = ...,
        url: Optional[Any] = ...,
        headers: Optional[Any] = ...,
        data: Optional[Any] = ...,
        params: Optional[Any] = ...,
        auth_path: Optional[Any] = ...,
        stream_output: bool = ...,
    ) -> None: ...
    def prepare(self) -> Any: ...
    @property
    def body(self) -> Any: ...

class AWSPreparedRequest:
    method: Any = ...
    url: Any = ...
    headers: Any = ...
    body: Any = ...
    stream_output: Any = ...
    def __init__(
        self, method: Any, url: Any, headers: Any, body: Any, stream_output: Any
    ) -> None: ...
    def reset_stream(self) -> None: ...

class AWSResponse:
    url: Any = ...
    status_code: Any = ...
    headers: Any = ...
    raw: Any = ...
    def __init__(self, url: Any, status_code: Any, headers: Any, raw: Any) -> None: ...
    @property
    def content(self) -> Any: ...
    @property
    def text(self) -> Any: ...

class _HeaderKey:
    def __init__(self, key: Any) -> None: ...
    def __hash__(self) -> Any: ...
    def __eq__(self, other: Any) -> Any: ...

class HeadersDict(MutableMapping[str, str]):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __delitem__(self, key: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def copy(self) -> Any: ...



from typing import Any

from botocore.config import Config as Config
from botocore.endpoint import EndpointCreator as EndpointCreator
from botocore.loaders import Loader
from botocore.signers import RequestSigner as RequestSigner

VALID_REGIONAL_ENDPOINTS_CONFIG: Any
LEGACY_GLOBAL_STS_REGIONS: Any

class ClientArgsCreator:
    def __init__(
        self,
        event_emitter: Any,
        user_agent: Any,
        response_parser_factory: Any,
        loader: Loader,
        exceptions_factory: Any,
        config_store: Any,
    ) -> None: ...
    def get_client_args(
        self,
        service_model: Any,
        region_name: Any,
        is_secure: Any,
        endpoint_url: Any,
        verify: Any,
        credentials: Any,
        scoped_config: Any,
        client_config: Any,
        endpoint_bridge: Any,
    ) -> Any: ...
    def compute_client_args(
        self,
        service_model: Any,
        client_config: Any,
        endpoint_bridge: Any,
        region_name: Any,
        endpoint_url: Any,
        is_secure: Any,
        scoped_config: Any,
    ) -> Any: ...
    def compute_s3_config(self, client_config: Any) -> Any: ...
